@@ -1,4 +1,3 @@
-# encoding: UTF-8
 require 'mkmf'
 require 'English'
 
@@ -13,7 +12,7 @@ def asplode(lib)
 end
 
 def add_ssl_defines(header)
-  all_modes_found = %w(SSL_MODE_DISABLED SSL_MODE_PREFERRED SSL_MODE_REQUIRED SSL_MODE_VERIFY_CA SSL_MODE_VERIFY_IDENTITY).inject(true) do |m, ssl_mode|
+  all_modes_found = %w[SSL_MODE_DISABLED SSL_MODE_PREFERRED SSL_MODE_REQUIRED SSL_MODE_VERIFY_CA SSL_MODE_VERIFY_IDENTITY].inject(true) do |m, ssl_mode|
     m && have_const(ssl_mode, header)
   end
   $CFLAGS << ' -DFULL_SSL_MODE_SUPPORT' if all_modes_found
@@ -26,19 +25,12 @@ end
 have_func('rb_absint_size')
 have_func('rb_absint_singlebit_p')
 
-# 2.0-only
-have_header('ruby/thread.h') && have_func('rb_thread_call_without_gvl', 'ruby/thread.h')
-
-# 1.9-only
-have_func('rb_thread_blocking_region')
+# Missing in RBX (https://github.com/rubinius/rubinius/issues/3771)
 have_func('rb_wait_for_single_fd')
-have_func('rb_hash_dup')
-have_func('rb_intern3')
-have_func('rb_big_cmp')
 
 # borrowed from mysqlplus
 # http://github.com/oldmoe/mysqlplus/blob/master/ext/extconf.rb
-dirs = ENV.fetch('PATH').split(File::PATH_SEPARATOR) + %w(
+dirs = ENV.fetch('PATH').split(File::PATH_SEPARATOR) + %w[
   /opt
   /opt/local
   /opt/local/mysql
@@ -50,20 +42,19 @@ dirs = ENV.fetch('PATH').split(File::PATH_SEPARATOR) + %w(
   /usr/local/mysql-*
   /usr/local/lib/mysql5*
   /usr/local/opt/mysql5*
-).map { |dir| dir << '/bin' }
+].map { |dir| dir << '/bin' }
 
 # For those without HOMEBREW_ROOT in PATH
 dirs << "#{ENV['HOMEBREW_ROOT']}/bin" if ENV['HOMEBREW_ROOT']
 
-GLOB = "{#{dirs.join(',')}}/{mysql_config,mysql_config5,mariadb_config}"
+GLOB = "{#{dirs.join(',')}}/{mysql_config,mysql_config5,mariadb_config}".freeze
 
 # If the user has provided a --with-mysql-dir argument, we must respect it or fail.
 inc, lib = dir_config('mysql')
 if inc && lib
-  # TODO: Remove when 2.0.0 is the minimum supported version
-  # Ruby versions not incorporating the mkmf fix at
+  # Ruby versions below 2.0 on Unix and below 2.1 on Windows
+  # do not properly search for lib directories, and must be corrected:
   # https://bugs.ruby-lang.org/projects/ruby-trunk/repository/revisions/39717
-  # do not properly search for lib directories, and must be corrected
   unless lib && lib[-3, 3] == 'lib'
     @libdir_basename = 'lib'
     inc, lib = dir_config('mysql')
@@ -72,6 +63,7 @@ if inc && lib
   abort "-----\nCannot find library dir(s) #{lib}\n-----" unless lib && lib.split(File::PATH_SEPARATOR).any? { |dir| File.directory?(dir) }
   warn "-----\nUsing --with-mysql-dir=#{File.dirname inc}\n-----"
   rpath_dir = lib
+  have_library('mysqlclient')
 elsif (mc = (with_config('mysql-config') || Dir[GLOB].first))
   # If the user has provided a --with-mysql-config argument, we must respect it or fail.
   # If the user gave --with-mysql-config with no argument means we should try to find it.
@@ -92,7 +84,7 @@ elsif (mc = (with_config('mysql-config') || Dir[GLOB].first))
 else
   _, usr_local_lib = dir_config('mysql', '/usr/local')
 
-  asplode("mysql client") unless find_library('mysqlclient', 'mysql_query', usr_local_lib, "#{usr_local_lib}/mysql")
+  asplode("mysql client") unless find_library('mysqlclient', nil, usr_local_lib, "#{usr_local_lib}/mysql")
 
   rpath_dir = usr_local_lib
 end
@@ -105,7 +97,7 @@ else
   asplode 'mysql.h'
 end
 
-%w(errmsg.h).each do |h|
+%w[errmsg.h].each do |h|
   header = [prefix, h].compact.join('/')
   asplode h unless have_header header
 end
@@ -114,7 +106,18 @@ mysql_h = [prefix, 'mysql.h'].compact.join('/')
 add_ssl_defines(mysql_h)
 have_struct_member('MYSQL', 'net.vio', mysql_h)
 have_struct_member('MYSQL', 'net.pvio', mysql_h)
+
+# These constants are actually enums, so they cannot be detected by #ifdef in C code.
 have_const('MYSQL_ENABLE_CLEARTEXT_PLUGIN', mysql_h)
+have_const('SERVER_QUERY_NO_GOOD_INDEX_USED', mysql_h)
+have_const('SERVER_QUERY_NO_INDEX_USED', mysql_h)
+have_const('SERVER_QUERY_WAS_SLOW', mysql_h)
+have_const('MYSQL_OPTION_MULTI_STATEMENTS_ON', mysql_h)
+have_const('MYSQL_OPTION_MULTI_STATEMENTS_OFF', mysql_h)
+
+# my_bool is replaced by C99 bool in MySQL 8.0, but we want
+# to retain compatibility with the typedef in earlier MySQLs.
+have_type('my_bool', mysql_h)
 
 # This is our wishlist. We use whichever flags work on the host.
 # -Wall and -Wextra are included by default.
@@ -150,7 +153,7 @@ sanitizers = with_config('sanitize')
 case sanitizers
 when true
   # Try them all, turn on whatever we can
-  enabled_sanitizers = %w(address cfi integer memory thread undefined).select do |s|
+  enabled_sanitizers = %w[address cfi integer memory thread undefined].select do |s|
     try_link('int main() {return 0;}',  "-Werror -fsanitize=#{s}")
   end
   abort "-----\nCould not enable any sanitizers!\n-----" if enabled_sanitizers.empty?
@@ -178,7 +181,7 @@ unless enabled_sanitizers.empty?
   $CFLAGS << ' -g -fno-omit-frame-pointer'
 end
 
-if RUBY_PLATFORM =~ /mswin|mingw/
+if RUBY_PLATFORM =~ /mswin|mingw/ && !defined?(RubyInstaller)
   # Build libmysql.a interface link library
   require 'rake'
 
