@@ -1,12 +1,15 @@
-# encoding: UTF-8
 require './spec/spec_helper.rb'
 
 RSpec.describe Mysql2::Statement do
   before :each do
-    @client = new_client(:encoding => "utf8")
+    @client = new_client(encoding: "utf8")
   end
 
   def stmt_count
+    # Use the performance schema in MySQL 5.7 and above
+    @client.query("SELECT COUNT(1) AS count FROM performance_schema.prepared_statements_instances").first['count'].to_i
+  rescue Mysql2::Error
+    # Fall back to the global prepapred statement counter
     @client.query("SHOW STATUS LIKE 'Prepared_stmt_count'").first['Value'].to_i
   end
 
@@ -87,6 +90,20 @@ RSpec.describe Mysql2::Statement do
     expect(result.to_a).to eq(['max1' => int64_max1, 'max2' => int64_max2, 'max3' => int64_max3, 'min1' => int64_min1, 'min2' => int64_min2, 'min3' => int64_min3])
   end
 
+  it "should accept keyword arguments on statement execute" do
+    stmt = @client.prepare 'SELECT 1 AS a'
+
+    expect(stmt.execute(as: :hash).first).to eq("a" => 1)
+    expect(stmt.execute(as: :array).first).to eq([1])
+  end
+
+  it "should accept bind arguments and keyword arguments on statement execute" do
+    stmt = @client.prepare 'SELECT ? AS a'
+
+    expect(stmt.execute(1, as: :hash).first).to eq("a" => 1)
+    expect(stmt.execute(1, as: :array).first).to eq([1])
+  end
+
   it "should keep its result after other query" do
     @client.query 'USE test'
     @client.query 'CREATE TABLE IF NOT EXISTS mysql2_stmt_q(a int)'
@@ -147,24 +164,16 @@ RSpec.describe Mysql2::Statement do
     now = Time.now
     statement = @client.prepare('SELECT ? AS a')
     result = statement.execute(now)
-    if RUBY_VERSION =~ /1.8/
-      expect(result.first['a'].strftime('%F %T %z')).to eql(now.strftime('%F %T %z'))
-    else
-      # microseconds is six digits after the decimal, but only test on 5 significant figures
-      expect(result.first['a'].strftime('%F %T.%5N %z')).to eql(now.strftime('%F %T.%5N %z'))
-    end
+    # microseconds is six digits after the decimal, but only test on 5 significant figures
+    expect(result.first['a'].strftime('%F %T.%5N %z')).to eql(now.strftime('%F %T.%5N %z'))
   end
 
   it "should prepare DateTime values with microseconds" do
     now = DateTime.now
     statement = @client.prepare('SELECT ? AS a')
     result = statement.execute(now)
-    if RUBY_VERSION =~ /1.8/
-      expect(result.first['a'].strftime('%F %T %z')).to eql(now.strftime('%F %T %z'))
-    else
-      # microseconds is six digits after the decimal, but only test on 5 significant figures
-      expect(result.first['a'].strftime('%F %T.%5N %z')).to eql(now.strftime('%F %T.%5N %z'))
-    end
+    # microseconds is six digits after the decimal, but only test on 5 significant figures
+    expect(result.first['a'].strftime('%F %T.%5N %z')).to eql(now.strftime('%F %T.%5N %z'))
   end
 
   it "should tell us about the fields" do
@@ -178,7 +187,7 @@ RSpec.describe Mysql2::Statement do
 
   it "should handle as a decimal binding a BigDecimal" do
     stmt = @client.prepare('SELECT ? AS decimal_test')
-    test_result = stmt.execute(BigDecimal.new("123.45")).first
+    test_result = stmt.execute(BigDecimal("123.45")).first
     expect(test_result['decimal_test']).to be_an_instance_of(BigDecimal)
     expect(test_result['decimal_test']).to eql(123.45)
   end
@@ -188,17 +197,16 @@ RSpec.describe Mysql2::Statement do
     @client.query 'DROP TABLE IF EXISTS mysql2_stmt_decimal_test'
     @client.query 'CREATE TABLE mysql2_stmt_decimal_test (decimal_test DECIMAL(10,3))'
 
-    @client.prepare("INSERT INTO mysql2_stmt_decimal_test VALUES (?)").execute(BigDecimal.new("123.45"))
+    @client.prepare("INSERT INTO mysql2_stmt_decimal_test VALUES (?)").execute(BigDecimal("123.45"))
 
     test_result = @client.query("SELECT * FROM mysql2_stmt_decimal_test").first
     expect(test_result['decimal_test']).to eql(123.45)
   end
 
   it "should warn but still work if cache_rows is set to false" do
-    @client.query_options.merge!(:cache_rows => false)
     statement = @client.prepare 'SELECT 1'
     result = nil
-    expect { result = statement.execute.to_a }.to output(/:cache_rows is forced for prepared statements/).to_stderr
+    expect { result = statement.execute(cache_rows: false).to_a }.to output(/:cache_rows is forced for prepared statements/).to_stderr
     expect(result.length).to eq(1)
   end
 
@@ -217,7 +225,7 @@ RSpec.describe Mysql2::Statement do
 
     it "should be able to retrieve utf8 field names correctly" do
       stmt = @client.prepare 'SELECT * FROM `テーブル`'
-      expect(stmt.fields).to eq(%w(整数 文字列))
+      expect(stmt.fields).to eq(%w[整数 文字列])
       result = stmt.execute
 
       expect(result.to_a).to eq([{ "整数" => 1, "文字列" => "イチ" }, { "整数" => 2, "文字列" => "弐" }, { "整数" => 3, "文字列" => "さん" }])
@@ -241,16 +249,13 @@ RSpec.describe Mysql2::Statement do
 
       expect(result.to_a).to eq([{ "整数" => 1 }])
     end
-  end if defined? Encoding
+  end
 
   context "streaming result" do
     it "should be able to stream query result" do
       n = 1
       stmt = @client.prepare("SELECT 1 UNION SELECT 2")
-
-      @client.query_options.merge!(:stream => true, :cache_rows => false, :as => :array)
-
-      stmt.execute.each do |r|
+      stmt.execute(stream: true, cache_rows: false, as: :array).each do |r|
         case n
         when 1
           expect(r).to eq([1])
@@ -276,23 +281,17 @@ RSpec.describe Mysql2::Statement do
     end
 
     it "should yield rows as hash's with symbol keys if :symbolize_keys was set to true" do
-      @client.query_options[:symbolize_keys] = true
-      @result = @client.prepare("SELECT 1").execute
+      @result = @client.prepare("SELECT 1").execute(symbolize_keys: true)
       @result.each do |row|
         expect(row.keys.first).to be_an_instance_of(Symbol)
       end
-      @client.query_options[:symbolize_keys] = false
     end
 
     it "should be able to return results as an array" do
-      @client.query_options[:as] = :array
-
-      @result = @client.prepare("SELECT 1").execute
+      @result = @client.prepare("SELECT 1").execute(as: :array)
       @result.each do |row|
         expect(row).to be_an_instance_of(Array)
       end
-
-      @client.query_options[:as] = :hash
     end
 
     it "should cache previously yielded results by default" do
@@ -301,35 +300,21 @@ RSpec.describe Mysql2::Statement do
     end
 
     it "should yield different value for #first if streaming" do
-      @client.query_options[:stream] = true
-      @client.query_options[:cache_rows] = false
-
-      result = @client.prepare("SELECT 1 UNION SELECT 2").execute
+      result = @client.prepare("SELECT 1 UNION SELECT 2").execute(stream: true, cache_rows: true)
       expect(result.first).not_to eql(result.first)
-
-      @client.query_options[:stream] = false
-      @client.query_options[:cache_rows] = true
     end
 
     it "should yield the same value for #first if streaming is disabled" do
-      @client.query_options[:stream] = false
-      result = @client.prepare("SELECT 1 UNION SELECT 2").execute
+      result = @client.prepare("SELECT 1 UNION SELECT 2").execute(stream: false)
       expect(result.first).to eql(result.first)
     end
 
     it "should throw an exception if we try to iterate twice when streaming is enabled" do
-      @client.query_options[:stream] = true
-      @client.query_options[:cache_rows] = false
-
-      result = @client.prepare("SELECT 1 UNION SELECT 2").execute
-
-      expect {
+      result = @client.prepare("SELECT 1 UNION SELECT 2").execute(stream: true, cache_rows: false)
+      expect do
         result.each {}
         result.each {}
-      }.to raise_exception(Mysql2::Error)
-
-      @client.query_options[:stream] = false
-      @client.query_options[:cache_rows] = true
+      end.to raise_exception(Mysql2::Error)
     end
   end
 
@@ -341,7 +326,7 @@ RSpec.describe Mysql2::Statement do
 
     it "should return an array of field names in proper order" do
       stmt = @client.prepare("SELECT 'a', 'b', 'c'")
-      expect(stmt.fields).to eql(%w(a b c))
+      expect(stmt.fields).to eql(%w[a b c])
     end
 
     it "should return nil for statement with no result fields" do
@@ -351,48 +336,44 @@ RSpec.describe Mysql2::Statement do
   end
 
   context "row data type mapping" do
-    before(:each) do
-      @client.query "USE test"
-      @test_result = @client.prepare("SELECT * FROM mysql2_test ORDER BY id DESC LIMIT 1").execute.first
-    end
+    let(:test_result) { @client.prepare("SELECT * FROM mysql2_test ORDER BY id DESC LIMIT 1").execute.first }
 
     it "should return nil for a NULL value" do
-      expect(@test_result['null_test']).to be_an_instance_of(NilClass)
-      expect(@test_result['null_test']).to eql(nil)
+      expect(test_result['null_test']).to be_an_instance_of(NilClass)
+      expect(test_result['null_test']).to eql(nil)
     end
 
     it "should return String for a BIT(64) value" do
-      expect(@test_result['bit_test']).to be_an_instance_of(String)
-      expect(@test_result['bit_test']).to eql("\000\000\000\000\000\000\000\005")
+      expect(test_result['bit_test']).to be_an_instance_of(String)
+      expect(test_result['bit_test']).to eql("\000\000\000\000\000\000\000\005")
     end
 
     it "should return String for a BIT(1) value" do
-      expect(@test_result['single_bit_test']).to be_an_instance_of(String)
-      expect(@test_result['single_bit_test']).to eql("\001")
+      expect(test_result['single_bit_test']).to be_an_instance_of(String)
+      expect(test_result['single_bit_test']).to eql("\001")
     end
 
     it "should return Fixnum for a TINYINT value" do
-      expect([Fixnum, Bignum]).to include(@test_result['tiny_int_test'].class)
-      expect(@test_result['tiny_int_test']).to eql(1)
+      expect(num_classes).to include(test_result['tiny_int_test'].class)
+      expect(test_result['tiny_int_test']).to eql(1)
     end
 
     context "cast booleans for TINYINT if :cast_booleans is enabled" do
       # rubocop:disable Style/Semicolon
-      let(:client) { new_client(:cast_booleans => true) }
-      let(:id1) { client.query 'INSERT INTO mysql2_test (bool_cast_test) VALUES ( 1)'; client.last_id }
-      let(:id2) { client.query 'INSERT INTO mysql2_test (bool_cast_test) VALUES ( 0)'; client.last_id }
-      let(:id3) { client.query 'INSERT INTO mysql2_test (bool_cast_test) VALUES (-1)'; client.last_id }
+      let(:id1) { @client.query 'INSERT INTO mysql2_test (bool_cast_test) VALUES ( 1)'; @client.last_id }
+      let(:id2) { @client.query 'INSERT INTO mysql2_test (bool_cast_test) VALUES ( 0)'; @client.last_id }
+      let(:id3) { @client.query 'INSERT INTO mysql2_test (bool_cast_test) VALUES (-1)'; @client.last_id }
       # rubocop:enable Style/Semicolon
 
       after do
-        client.query "DELETE from mysql2_test WHERE id IN(#{id1},#{id2},#{id3})"
+        @client.query "DELETE from mysql2_test WHERE id IN(#{id1},#{id2},#{id3})"
       end
 
       it "should return TrueClass or FalseClass for a TINYINT value if :cast_booleans is enabled" do
-        query = client.prepare 'SELECT bool_cast_test FROM mysql2_test WHERE id = ?'
-        result1 = query.execute id1
-        result2 = query.execute id2
-        result3 = query.execute id3
+        query = @client.prepare 'SELECT bool_cast_test FROM mysql2_test WHERE id = ?'
+        result1 = query.execute id1, cast_booleans: true
+        result2 = query.execute id2, cast_booleans: true
+        result3 = query.execute id3, cast_booleans: true
         expect(result1.first['bool_cast_test']).to be true
         expect(result2.first['bool_cast_test']).to be false
         expect(result3.first['bool_cast_test']).to be true
@@ -401,134 +382,96 @@ RSpec.describe Mysql2::Statement do
 
     context "cast booleans for BIT(1) if :cast_booleans is enabled" do
       # rubocop:disable Style/Semicolon
-      let(:client) { new_client(:cast_booleans => true) }
-      let(:id1) { client.query 'INSERT INTO mysql2_test (single_bit_test) VALUES (1)'; client.last_id }
-      let(:id2) { client.query 'INSERT INTO mysql2_test (single_bit_test) VALUES (0)'; client.last_id }
+      let(:id1) { @client.query 'INSERT INTO mysql2_test (single_bit_test) VALUES (1)'; @client.last_id }
+      let(:id2) { @client.query 'INSERT INTO mysql2_test (single_bit_test) VALUES (0)'; @client.last_id }
       # rubocop:enable Style/Semicolon
 
       after do
-        client.query "DELETE from mysql2_test WHERE id IN(#{id1},#{id2})"
+        @client.query "DELETE from mysql2_test WHERE id IN(#{id1},#{id2})"
       end
 
       it "should return TrueClass or FalseClass for a BIT(1) value if :cast_booleans is enabled" do
-        query = client.prepare 'SELECT single_bit_test FROM mysql2_test WHERE id = ?'
-        result1 = query.execute id1
-        result2 = query.execute id2
+        query = @client.prepare 'SELECT single_bit_test FROM mysql2_test WHERE id = ?'
+        result1 = query.execute id1, cast_booleans: true
+        result2 = query.execute id2, cast_booleans: true
         expect(result1.first['single_bit_test']).to be true
         expect(result2.first['single_bit_test']).to be false
       end
     end
 
     it "should return Fixnum for a SMALLINT value" do
-      expect([Fixnum, Bignum]).to include(@test_result['small_int_test'].class)
-      expect(@test_result['small_int_test']).to eql(10)
+      expect(num_classes).to include(test_result['small_int_test'].class)
+      expect(test_result['small_int_test']).to eql(10)
     end
 
     it "should return Fixnum for a MEDIUMINT value" do
-      expect([Fixnum, Bignum]).to include(@test_result['medium_int_test'].class)
-      expect(@test_result['medium_int_test']).to eql(10)
+      expect(num_classes).to include(test_result['medium_int_test'].class)
+      expect(test_result['medium_int_test']).to eql(10)
     end
 
     it "should return Fixnum for an INT value" do
-      expect([Fixnum, Bignum]).to include(@test_result['int_test'].class)
-      expect(@test_result['int_test']).to eql(10)
+      expect(num_classes).to include(test_result['int_test'].class)
+      expect(test_result['int_test']).to eql(10)
     end
 
     it "should return Fixnum for a BIGINT value" do
-      expect([Fixnum, Bignum]).to include(@test_result['big_int_test'].class)
-      expect(@test_result['big_int_test']).to eql(10)
+      expect(num_classes).to include(test_result['big_int_test'].class)
+      expect(test_result['big_int_test']).to eql(10)
     end
 
     it "should return Fixnum for a YEAR value" do
-      expect([Fixnum, Bignum]).to include(@test_result['year_test'].class)
-      expect(@test_result['year_test']).to eql(2009)
+      expect(num_classes).to include(test_result['year_test'].class)
+      expect(test_result['year_test']).to eql(2009)
     end
 
     it "should return BigDecimal for a DECIMAL value" do
-      expect(@test_result['decimal_test']).to be_an_instance_of(BigDecimal)
-      expect(@test_result['decimal_test']).to eql(10.3)
+      expect(test_result['decimal_test']).to be_an_instance_of(BigDecimal)
+      expect(test_result['decimal_test']).to eql(10.3)
     end
 
     it "should return Float for a FLOAT value" do
-      expect(@test_result['float_test']).to be_an_instance_of(Float)
-      expect(@test_result['float_test']).to be_within(1e-5).of(10.3)
+      expect(test_result['float_test']).to be_an_instance_of(Float)
+      expect(test_result['float_test']).to be_within(1e-5).of(10.3)
     end
 
     it "should return Float for a DOUBLE value" do
-      expect(@test_result['double_test']).to be_an_instance_of(Float)
-      expect(@test_result['double_test']).to eql(10.3)
+      expect(test_result['double_test']).to be_an_instance_of(Float)
+      expect(test_result['double_test']).to eql(10.3)
     end
 
     it "should return Time for a DATETIME value when within the supported range" do
-      expect(@test_result['date_time_test']).to be_an_instance_of(Time)
-      expect(@test_result['date_time_test'].strftime("%Y-%m-%d %H:%M:%S")).to eql('2010-04-04 11:44:00')
+      expect(test_result['date_time_test']).to be_an_instance_of(Time)
+      expect(test_result['date_time_test'].strftime("%Y-%m-%d %H:%M:%S")).to eql('2010-04-04 11:44:00')
     end
 
-    if 1.size == 4 # 32bit
-      klass = if RUBY_VERSION =~ /1.8/
-        DateTime
-      else
-        Time
-      end
+    it "should return Time when timestamp is < 1901-12-13 20:45:52" do
+      r = @client.prepare("SELECT CAST('1901-12-13 20:45:51' AS DATETIME) as test").execute
+      expect(r.first['test']).to be_an_instance_of(Time)
+    end
 
-      it "should return DateTime when timestamp is < 1901-12-13 20:45:52" do
-        # 1901-12-13T20:45:52 is the min for 32bit Ruby 1.8
-        r = @client.prepare("SELECT CAST('1901-12-13 20:45:51' AS DATETIME) as test").execute
-        expect(r.first['test']).to be_an_instance_of(klass)
-      end
-
-      it "should return DateTime when timestamp is > 2038-01-19T03:14:07" do
-        # 2038-01-19T03:14:07 is the max for 32bit Ruby 1.8
-        r = @client.prepare("SELECT CAST('2038-01-19 03:14:08' AS DATETIME) as test").execute
-        expect(r.first['test']).to be_an_instance_of(klass)
-      end
-    elsif 1.size == 8 # 64bit
-      if RUBY_VERSION =~ /1.8/
-        it "should return Time when timestamp is > 0138-12-31 11:59:59" do
-          r = @client.prepare("SELECT CAST('0139-1-1 00:00:00' AS DATETIME) as test").execute
-          expect(r.first['test']).to be_an_instance_of(Time)
-        end
-
-        it "should return DateTime when timestamp is < 0139-1-1T00:00:00" do
-          r = @client.prepare("SELECT CAST('0138-12-31 11:59:59' AS DATETIME) as test").execute
-          expect(r.first['test']).to be_an_instance_of(DateTime)
-        end
-
-        it "should return Time when timestamp is > 2038-01-19T03:14:07" do
-          r = @client.prepare("SELECT CAST('2038-01-19 03:14:08' AS DATETIME) as test").execute
-          expect(r.first['test']).to be_an_instance_of(Time)
-        end
-      else
-        it "should return Time when timestamp is < 1901-12-13 20:45:52" do
-          r = @client.prepare("SELECT CAST('1901-12-13 20:45:51' AS DATETIME) as test").execute
-          expect(r.first['test']).to be_an_instance_of(Time)
-        end
-
-        it "should return Time when timestamp is > 2038-01-19T03:14:07" do
-          r = @client.prepare("SELECT CAST('2038-01-19 03:14:08' AS DATETIME) as test").execute
-          expect(r.first['test']).to be_an_instance_of(Time)
-        end
-      end
+    it "should return Time when timestamp is > 2038-01-19T03:14:07" do
+      r = @client.prepare("SELECT CAST('2038-01-19 03:14:08' AS DATETIME) as test").execute
+      expect(r.first['test']).to be_an_instance_of(Time)
     end
 
     it "should return Time for a TIMESTAMP value when within the supported range" do
-      expect(@test_result['timestamp_test']).to be_an_instance_of(Time)
-      expect(@test_result['timestamp_test'].strftime("%Y-%m-%d %H:%M:%S")).to eql('2010-04-04 11:44:00')
+      expect(test_result['timestamp_test']).to be_an_instance_of(Time)
+      expect(test_result['timestamp_test'].strftime("%Y-%m-%d %H:%M:%S")).to eql('2010-04-04 11:44:00')
     end
 
     it "should return Time for a TIME value" do
-      expect(@test_result['time_test']).to be_an_instance_of(Time)
-      expect(@test_result['time_test'].strftime("%Y-%m-%d %H:%M:%S")).to eql('2000-01-01 11:44:00')
+      expect(test_result['time_test']).to be_an_instance_of(Time)
+      expect(test_result['time_test'].strftime("%Y-%m-%d %H:%M:%S")).to eql('2000-01-01 11:44:00')
     end
 
     it "should return Date for a DATE value" do
-      expect(@test_result['date_test']).to be_an_instance_of(Date)
-      expect(@test_result['date_test'].strftime("%Y-%m-%d")).to eql('2010-04-04')
+      expect(test_result['date_test']).to be_an_instance_of(Date)
+      expect(test_result['date_test'].strftime("%Y-%m-%d")).to eql('2010-04-04')
     end
 
     it "should return String for an ENUM value" do
-      expect(@test_result['enum_test']).to be_an_instance_of(String)
-      expect(@test_result['enum_test']).to eql('val1')
+      expect(test_result['enum_test']).to be_an_instance_of(String)
+      expect(test_result['enum_test']).to eql('val1')
     end
 
     it "should raise an error given an invalid DATETIME" do
@@ -537,14 +480,12 @@ RSpec.describe Mysql2::Statement do
     end
 
     context "string encoding for ENUM values" do
-      before { pending('Encoding is undefined') unless defined?(Encoding) }
-
       it "should default to the connection's encoding if Encoding.default_internal is nil" do
         with_internal_encoding nil do
           result = @client.query("SELECT * FROM mysql2_test ORDER BY id DESC LIMIT 1").first
           expect(result['enum_test'].encoding).to eql(Encoding::UTF_8)
 
-          client2 = new_client(:encoding => 'ascii')
+          client2 = new_client(encoding: 'ascii')
           result = client2.query("SELECT * FROM mysql2_test ORDER BY id DESC LIMIT 1").first
           expect(result['enum_test'].encoding).to eql(Encoding::US_ASCII)
         end
@@ -564,19 +505,17 @@ RSpec.describe Mysql2::Statement do
     end
 
     it "should return String for a SET value" do
-      expect(@test_result['set_test']).to be_an_instance_of(String)
-      expect(@test_result['set_test']).to eql('val1,val2')
+      expect(test_result['set_test']).to be_an_instance_of(String)
+      expect(test_result['set_test']).to eql('val1,val2')
     end
 
     context "string encoding for SET values" do
-      before { pending('Encoding is undefined') unless defined?(Encoding) }
-
       it "should default to the connection's encoding if Encoding.default_internal is nil" do
         with_internal_encoding nil do
           result = @client.query("SELECT * FROM mysql2_test ORDER BY id DESC LIMIT 1").first
           expect(result['set_test'].encoding).to eql(Encoding::UTF_8)
 
-          client2 = new_client(:encoding => 'ascii')
+          client2 = new_client(encoding: 'ascii')
           result = client2.query("SELECT * FROM mysql2_test ORDER BY id DESC LIMIT 1").first
           expect(result['set_test'].encoding).to eql(Encoding::US_ASCII)
         end
@@ -596,13 +535,11 @@ RSpec.describe Mysql2::Statement do
     end
 
     it "should return String for a BINARY value" do
-      expect(@test_result['binary_test']).to be_an_instance_of(String)
-      expect(@test_result['binary_test']).to eql("test#{"\000" * 6}")
+      expect(test_result['binary_test']).to be_an_instance_of(String)
+      expect(test_result['binary_test']).to eql("test#{"\000" * 6}")
     end
 
     context "string encoding for BINARY values" do
-      before { pending('Encoding is undefined') unless defined?(Encoding) }
-
       it "should default to binary if Encoding.default_internal is nil" do
         with_internal_encoding nil do
           result = @client.query("SELECT * FROM mysql2_test ORDER BY id DESC LIMIT 1").first
@@ -637,14 +574,12 @@ RSpec.describe Mysql2::Statement do
       'long_text_test' => 'LONGTEXT',
     }.each do |field, type|
       it "should return a String for #{type}" do
-        expect(@test_result[field]).to be_an_instance_of(String)
-        expect(@test_result[field]).to eql("test")
+        expect(test_result[field]).to be_an_instance_of(String)
+        expect(test_result[field]).to eql("test")
       end
 
       context "string encoding for #{type} values" do
-        before { pending('Encoding is undefined') unless defined?(Encoding) }
-
-        if %w(VARBINARY TINYBLOB BLOB MEDIUMBLOB LONGBLOB).include?(type)
+        if %w[VARBINARY TINYBLOB BLOB MEDIUMBLOB LONGBLOB].include?(type)
           it "should default to binary if Encoding.default_internal is nil" do
             with_internal_encoding nil do
               result = @client.query("SELECT * FROM mysql2_test ORDER BY id DESC LIMIT 1").first
@@ -669,7 +604,7 @@ RSpec.describe Mysql2::Statement do
               result = @client.query("SELECT * FROM mysql2_test ORDER BY id DESC LIMIT 1").first
               expect(result[field].encoding).to eql(Encoding::UTF_8)
 
-              client2 = new_client(:encoding => 'ascii')
+              client2 = new_client(encoding: 'ascii')
               result = client2.query("SELECT * FROM mysql2_test ORDER BY id DESC LIMIT 1").first
               expect(result[field].encoding).to eql(Encoding::US_ASCII)
             end
@@ -731,7 +666,6 @@ RSpec.describe Mysql2::Statement do
 
     it 'should return number of rows affected by an insert' do
       stmt = @client.prepare 'INSERT INTO lastIdTest (blah) VALUES (?)'
-      expect(stmt.affected_rows).to eq 0
       stmt.execute 1
       expect(stmt.affected_rows).to eq 1
     end
@@ -764,7 +698,9 @@ RSpec.describe Mysql2::Statement do
   context 'close' do
     it 'should free server resources' do
       stmt = @client.prepare 'SELECT 1'
+      GC.disable
       expect { stmt.close }.to change(&method(:stmt_count)).by(-1)
+      GC.enable
     end
 
     it 'should raise an error on subsequent execution' do
